<?php

namespace Drupal\Tests\webform_analysis\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\webform\Entity\Webform;

/**
 * Tests the Webform Analysis module functionality.
 *
 * @group webform_analysis
 */
class WebformAnalysisTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = ['webform', 'webform_analysis'];

  /**
   * A user with permission to administer webforms.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create an admin user with webform permissions.
    $this->adminUser = $this->drupalCreateUser([
      'administer webform',
      'access webform overview',
      'view any webform submission',
      'edit any webform submission',
      'delete any webform submission',
    ]);
  }

  /**
   * Tests that the Analysis page is accessible for authorized users.
   */
  public function testAnalysisPageAccess() {
    // Create a test webform.
    $webform = Webform::create([
      'id' => 'test_webform',
      'title' => 'Test Webform',
      'elements' => "name:\n  '#type': textfield\n  '#title': 'Your Name'",
    ]);
    $webform->save();

    // Login as admin user.
    $this->drupalLogin($this->adminUser);

    // Test direct access to the analysis page.
    $this->drupalGet('/admin/structure/webform/manage/' . $webform->id() . '/results/analysis');

    // Verify the page loads successfully.
    $this->assertSession()->statusCodeEquals(200);

    // Check for expected elements on the analysis page.
    $this->assertSession()->pageTextContains('Test Webform');

    // Verify the form contains expected analysis elements.
    $this->assertSession()->fieldExists('analysis_chart_type');
  }

  /**
   * Tests that unauthorized users cannot access the Analysis page.
   */
  public function testAnalysisPageAccessControl() {
    // Create a test webform.
    $webform = Webform::create([
      'id' => 'test_webform_access',
      'title' => 'Test Webform Access',
      'elements' => "email:\n  '#type': email\n  '#title': 'Email Address'",
    ]);
    $webform->save();

    // Create a user without webform admin permissions.
    $limitedUser = $this->drupalCreateUser(['access content']);

    // Login as limited user.
    $this->drupalLogin($limitedUser);

    // Try to access the analysis page directly.
    $this->drupalGet('/admin/structure/webform/manage/' . $webform->id() . '/results/analysis');

    // Verify access is denied.
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests saving WebformAnalysisForm settings and verifying they persist.
   */
  public function testAnalysisFormSettings() {
    // Create a test webform with multiple elements.
    $webform = Webform::create([
      'id' => 'test_webform_settings',
      'title' => 'Test Webform Settings',
      'elements' => "name:\n  '#type': textfield\n  '#title': 'Your Name'\nemail:\n  '#type': email\n  '#title': 'Email Address'\nage:\n  '#type': number\n  '#title': 'Age'",
    ]);
    $webform->save();

    // Login as admin user.
    $this->drupalLogin($this->adminUser);

    // Visit the analysis page.
    $this->drupalGet('/admin/structure/webform/manage/' . $webform->id() . '/results/analysis');
    $this->assertSession()->statusCodeEquals(200);

    // Verify form elements are present.
    $this->assertSession()->fieldExists('analysis_chart_type');
    $this->assertSession()->fieldExists('analysis_components[name]');
    $this->assertSession()->fieldExists('analysis_components[email]');
    $this->assertSession()->fieldExists('analysis_components[age]');
    $this->assertSession()->fieldExists('in_draft');

    // Submit the form with specific settings.
    $edit = [
      'analysis_chart_type' => 'PieChart',
      'analysis_components[name]' => TRUE,
      'analysis_components[email]' => TRUE,
      'analysis_components[age]' => FALSE,
      'in_draft' => 1,
    ];
    $this->submitForm($edit, 'Update analysis display');

    // Verify the form was submitted successfully.
    $this->assertSession()->statusCodeEquals(200);

    // Clear entity cache and reload the webform entity to get saved settings.
    \Drupal::entityTypeManager()->getStorage('webform')
      ->resetCache(['test_webform_settings']);
    $webform = Webform::load('test_webform_settings');

    // Verify the chart type was saved correctly.
    $saved_chart_type = $webform->getThirdPartySetting('webform_analysis', 'chart_type');
    $this->assertEquals('PieChart', $saved_chart_type, 'Chart type should be saved as PieChart');

    // Verify the components were saved correctly.
    $saved_components = $webform->getThirdPartySetting('webform_analysis', 'components');
    $this->assertIsArray($saved_components, 'Components should be saved as an array');
    $this->assertContains('name', $saved_components, 'Name component should be selected');
    $this->assertContains('email', $saved_components, 'Email component should be selected');
    $this->assertNotContains('age', $saved_components, 'Age component should not be selected');

    // Verify the in_draft setting was saved correctly.
    $saved_in_draft = $webform->getThirdPartySetting('webform_analysis', 'in_draft');
    $this->assertTrue($saved_in_draft, 'In draft setting should be saved as TRUE');

    // Visit the analysis page again to verify the form shows saved values.
    $this->drupalGet('/admin/structure/webform/manage/' . $webform->id() . '/results/analysis');
    $this->assertSession()->statusCodeEquals(200);

    // Verify the form fields show the previously saved values.
    $this->assertSession()->fieldValueEquals('analysis_chart_type', 'PieChart');
    $this->assertSession()->checkboxChecked('analysis_components[name]');
    $this->assertSession()->checkboxChecked('analysis_components[email]');
    $this->assertSession()->checkboxNotChecked('analysis_components[age]');
    $this->assertSession()->checkboxChecked('in_draft');

    // Test saving different values to ensure updates work.
    $edit = [
      'analysis_chart_type' => 'ColumnChart',
      'analysis_components[name]' => FALSE,
      'analysis_components[email]' => TRUE,
      'analysis_components[age]' => TRUE,
      'in_draft' => 0,
    ];
    $this->submitForm($edit, 'Update analysis display');

    // Reload and verify the updated settings.
    \Drupal::entityTypeManager()->getStorage('webform')->resetCache(['test_webform_settings']);
    $webform = Webform::load('test_webform_settings');
    $this->assertEquals('ColumnChart', $webform->getThirdPartySetting('webform_analysis', 'chart_type'));

    $updated_components = $webform->getThirdPartySetting('webform_analysis', 'components');
    $this->assertNotContains('name', $updated_components, 'Name component should not be selected after update');
    $this->assertContains('email', $updated_components, 'Email component should still be selected');
    $this->assertContains('age', $updated_components, 'Age component should now be selected');

    $this->assertFalse($webform->getThirdPartySetting('webform_analysis', 'in_draft'), 'In draft should be FALSE after update');
  }

}
